=== modified file 'NEWS'
--- NEWS	2007-11-04 03:00:07 +0000
+++ NEWS	2007-11-04 15:29:17 +0000
@@ -30,6 +30,11 @@
 
   FEATURES:
 
+   * New ``authentication.conf`` file holding the password or other credentials
+     for remote servers. This can be used for ssh, sftp, smtp and other 
+     supported transports.
+     (Vincent Ladeuil)
+
    * New ``knitpack-experimental`` repository format. This is interoperable with
      the ``dirstate-tags`` format but uses a smarter storage design that greatly
      speeds up many operations, both local and remote. This new format can be

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2007-10-05 14:52:02 +0000
+++ bzrlib/config.py	2007-11-04 15:44:32 +0000
@@ -70,23 +70,23 @@
 import errno
 from fnmatch import fnmatch
 import re
-from StringIO import StringIO
+from cStringIO import StringIO
 
 import bzrlib
 from bzrlib import (
+    debug,
     errors,
     mail_client,
     osutils,
     symbol_versioning,
     trace,
+    ui,
     urlutils,
     win32utils,
     )
 import bzrlib.util.configobj.configobj as configobj
 """)
 
-from bzrlib.trace import mutter, warning
-
 
 CHECK_IF_POSSIBLE=0
 CHECK_ALWAYS=1
@@ -236,15 +236,11 @@
         v = os.environ.get('BZR_EMAIL')
         if v:
             return v.decode(bzrlib.user_encoding)
-        v = os.environ.get('BZREMAIL')
-        if v:
-            warning('BZREMAIL is deprecated in favor of BZR_EMAIL. Please update your configuration.')
-            return v.decode(bzrlib.user_encoding)
-    
+
         v = self._get_user_id()
         if v:
             return v
-        
+
         v = os.environ.get('EMAIL')
         if v:
             return v.decode(bzrlib.user_encoding)
@@ -275,8 +271,8 @@
         if policy is None:
             policy = self._get_signature_checking()
             if policy is not None:
-                warning("Please use create_signatures, not check_signatures "
-                        "to set signing policy.")
+                trace.warning("Please use create_signatures,"
+                              " not check_signatures to set signing policy.")
             if policy == CHECK_ALWAYS:
                 return True
         elif policy == SIGN_ALWAYS:
@@ -460,15 +456,15 @@
 
     def __init__(self, location):
         name_generator = locations_config_filename
-        if (not os.path.exists(name_generator()) and 
+        if (not os.path.exists(name_generator()) and
                 os.path.exists(branches_config_filename())):
             if sys.platform == 'win32':
-                warning('Please rename %s to %s' 
-                         % (branches_config_filename(),
-                            locations_config_filename()))
+                trace.warning('Please rename %s to %s'
+                              % (branches_config_filename(),
+                                 locations_config_filename()))
             else:
-                warning('Please rename ~/.bazaar/branches.conf'
-                        ' to ~/.bazaar/locations.conf')
+                trace.warning('Please rename ~/.bazaar/branches.conf'
+                              ' to ~/.bazaar/locations.conf')
             name_generator = branches_config_filename
         super(LocationConfig, self).__init__(name_generator)
         # local file locations are looked up by local path, rather than
@@ -743,9 +739,9 @@
         if sys.platform == 'win32':
             parent_dir = os.path.dirname(path)
             if not os.path.isdir(parent_dir):
-                mutter('creating config parent directory: %r', parent_dir)
+                trace.mutter('creating config parent directory: %r', parent_dir)
             os.mkdir(parent_dir)
-        mutter('creating config directory: %r', path)
+        trace.mutter('creating config directory: %r', path)
         os.mkdir(path)
 
 
@@ -787,6 +783,11 @@
     return osutils.pathjoin(config_dir(), 'locations.conf')
 
 
+def authentication_config_filename():
+    """Return per-user authentication ini file filename."""
+    return osutils.pathjoin(config_dir(), 'authentication.conf')
+
+
 def user_ignore_config_filename():
     """Return the user default ignore filename"""
     return osutils.pathjoin(config_dir(), 'ignore')
@@ -878,6 +879,7 @@
 
 class TreeConfig(IniBasedConfig):
     """Branch configuration data associated with its contents, not location"""
+
     def __init__(self, branch):
         self.branch = branch
 
@@ -928,3 +930,188 @@
             self.branch.control_files.put('branch.conf', out_file)
         finally:
             self.branch.unlock()
+
+
+class AuthenticationConfig(object):
+    """The authentication configuration file based on a ini file.
+
+    Implements the authentication.conf file described in
+    doc/developers/authentication-ring.txt.
+    """
+
+    def __init__(self, _file=None):
+        self._config = None # The ConfigObj
+        if _file is None:
+            self._filename = authentication_config_filename()
+            self._input = self._filename = authentication_config_filename()
+        else:
+            # Tests can provide a string as _file
+            self._filename = None
+            self._input = _file
+
+    def _get_config(self):
+        if self._config is not None:
+            return self._config
+        try:
+            # FIXME: Should we validate something here ? Includes: empty
+            # sections are useless, at least one of
+            # user/password/password_encoding should be defined, etc.
+
+            # Note: the encoding below declares that the file itself is utf-8
+            # encoded, but the values in the ConfigObj are always Unicode.
+            self._config = ConfigObj(self._input, encoding='utf-8')
+        except configobj.ConfigObjError, e:
+            raise errors.ParseConfigError(e.errors, e.config.filename)
+        return self._config
+
+    def _save(self):
+        """Save the config file, only tests should use it for now."""
+        conf_dir = os.path.dirname(self._filename)
+        ensure_config_dir_exists(conf_dir)
+        self._get_config().write(file(self._filename, 'wb'))
+
+    def _set_option(self, section_name, option_name, value):
+        """Set an authentication configuration option"""
+        conf = self._get_config()
+        section = conf.get(section_name)
+        if section is None:
+            conf[section] = {}
+            section = conf[section]
+        section[option_name] = value
+        self._save()
+
+    def get_credentials(self, scheme, host, port=None, user=None, path=None):
+        """Returns the matching credentials from authentication.conf file.
+
+        :param scheme: protocol
+
+        :param host: the server address
+
+        :param port: the associated port (optional)
+
+        :param user: login (optional)
+
+        :param path: the absolute path on the server (optional)
+
+        :return: A dict containing the matching credentials or None.
+           This includes:
+           - name: the section name of the credentials in the
+             authentication.conf file,
+           - user: can't de different from the provided user if any,
+           - password: the decoded password, could be None if the credential
+             defines only the user
+           - verify_certificates: https specific, True if the server
+             certificate should be verified, False otherwise.
+        """
+        credentials = None
+        for auth_def_name, auth_def in self._get_config().items():
+            a_scheme, a_host, a_user, a_path = map(
+                auth_def.get, ['scheme', 'host', 'user', 'path'])
+
+            try:
+                a_port = auth_def.as_int('port')
+            except KeyError:
+                a_port = None
+            except ValueError:
+                raise ValueError("'port' not numeric in %s" % auth_def_name)
+            try:
+                a_verify_certificates = auth_def.as_bool('verify_certificates')
+            except KeyError:
+                a_verify_certificates = True
+            except ValueError:
+                raise ValueError(
+                    "'verify_certificates' not boolean in %s" % auth_def_name)
+
+            # Attempt matching
+            if a_scheme is not None and scheme != a_scheme:
+                continue
+            if a_host is not None:
+                if not (host == a_host
+                        or (a_host.startswith('.') and host.endswith(a_host))):
+                    continue
+            if a_port is not None and port != a_port:
+                continue
+            if (a_path is not None and path is not None
+                and not path.startswith(a_path)):
+                continue
+            if (a_user is not None and user is not None
+                and a_user != user):
+                # Never contradict the caller about the user to be used
+                continue
+            if a_user is None:
+                # Can't find a user
+                continue
+            credentials = dict(name=auth_def_name,
+                               user=a_user, password=auth_def['password'],
+                               verify_certificates=a_verify_certificates)
+            self.decode_password(credentials,
+                                 auth_def.get('password_encoding', None))
+            if 'auth' in debug.debug_flags:
+                trace.mutter("Using authentication section: %r", auth_def_name)
+            break
+
+        return credentials
+
+    def get_user(self, scheme, host, port=None,
+                 realm=None, path=None, prompt=None):
+        """Get a user from authentication file.
+
+        :param scheme: protocol
+
+        :param host: the server address
+
+        :param port: the associated port (optional)
+
+        :param realm: the realm sent by the server (optional)
+
+        :param path: the absolute path on the server (optional)
+
+        :return: The found user.
+        """
+        credentials = self.get_credentials(scheme, host, port, user=None,
+                                           path=path)
+        if credentials is not None:
+            user = credentials['user']
+        else:
+            user = None
+        return user
+
+    def get_password(self, scheme, host, user, port=None,
+                     realm=None, path=None, prompt=None):
+        """Get a password from authentication file or prompt the user for one.
+
+        :param scheme: protocol
+
+        :param host: the server address
+
+        :param port: the associated port (optional)
+
+        :param user: login
+
+        :param realm: the realm sent by the server (optional)
+
+        :param path: the absolute path on the server (optional)
+
+        :return: The found password or the one entered by the user.
+        """
+        credentials = self.get_credentials(scheme, host, port, user, path)
+        if credentials is not None:
+            password = credentials['password']
+        else:
+            password = None
+        # Prompt user only if we could't find a password
+        if password is None:
+            if prompt is None:
+                # Create a default prompt suitable for most of the cases
+                prompt = '%s' % scheme.upper() + ' %(user)s@%(host)s password'
+            # Special handling for optional fields in the prompt
+            if port is not None:
+                prompt_host = '%s:%d' % (host, port)
+            else:
+                prompt_host = host
+            password = ui.ui_factory.get_password(prompt,
+                                                  host=prompt_host, user=user)
+        return password
+
+    def decode_password(self, credentials, encoding):
+        return credentials

=== modified file 'bzrlib/debug.py'
--- bzrlib/debug.py	2007-10-04 22:51:29 +0000
+++ bzrlib/debug.py	2007-11-04 15:29:17 +0000
@@ -25,8 +25,9 @@
 
 Options include:
  
+ * auth - show authentication sections used
+ * error - show stack traces for all top level exceptions
  * evil - capture call sites that do expensive or badly-scaling operations.
- * error - show stack traces for all top level exceptions
  * fetch - trace history copying between repositories
  * hooks - trace hook execution
  * hpss - trace smart protocol requests and responses

=== modified file 'bzrlib/help_topics.py'
--- bzrlib/help_topics.py	2007-09-28 07:09:05 +0000
+++ bzrlib/help_topics.py	2007-10-18 10:33:11 +0000
@@ -261,6 +261,7 @@
 A number of debug flags are also available to assist troubleshooting and
 development.
 
+-Dauth         Trace authentication sections used.
 -Derror        Instead of normal error handling, always print a traceback on
                error.
 -Devil         Capture call sites that do expensive or badly-scaling

=== modified file 'bzrlib/plugins/launchpad/lp_registration.py'
--- bzrlib/plugins/launchpad/lp_registration.py	2006-10-11 23:08:27 +0000
+++ bzrlib/plugins/launchpad/lp_registration.py	2007-10-23 14:31:10 +0000
@@ -21,8 +21,11 @@
 import urllib
 import xmlrpclib
 
-import bzrlib.config
-import bzrlib.errors as errors
+from bzrlib import (
+    config,
+    errors,
+    __version__ as _bzrlib_version,
+    )
 
 # for testing, do
 '''
@@ -53,7 +56,7 @@
             else:
                 transport = xmlrpclib.Transport()
             transport.user_agent = 'bzr/%s (xmlrpclib/%s)' \
-                    % (bzrlib.__version__, xmlrpclib.__version__)
+                    % (_bzrlib_version, xmlrpclib.__version__)
         self.transport = transport
 
 
@@ -92,12 +95,17 @@
 
     def gather_user_credentials(self):
         """Get the password from the user."""
-        config = bzrlib.config.GlobalConfig()
+        config = config.GlobalConfig()
         self.registrant_email = config.user_email()
         if self.registrant_password is None:
+            auth = config.AuthenticationConfig()
+            scheme, hostinfo = urlsplit(self.service_url)[:2]
             prompt = 'launchpad.net password for %s: ' % \
                     self.registrant_email
-            self.registrant_password = getpass(prompt)
+            # We will reuse http[s] credentials if we can, prompt user
+            # otherwise
+            self.registrant_password = auth.get_password(scheme, hostinfo,
+                                                         prompt=prompt)
 
     def send_request(self, method_name, method_params):
         proxy = self.get_proxy()

=== modified file 'bzrlib/smtp_connection.py'
--- bzrlib/smtp_connection.py	2007-10-20 00:14:37 +0000
+++ bzrlib/smtp_connection.py	2007-10-23 07:15:13 +0000
@@ -21,7 +21,10 @@
 import smtplib
 import socket
 
-from bzrlib import ui
+from bzrlib import (
+    config,
+    ui,
+    )
 from bzrlib.errors import (
     NoDestinationAddress,
     SMTPError,
@@ -98,14 +101,15 @@
 
     def _authenticate(self):
         """If necessary authenticate yourself to the server."""
+        auth = config.AuthenticationConfig()
         if self._smtp_username is None:
-            return
+            self._smtp_username = auth.get_user('smtp', self._smtp_server)
+            if self._smtp_username is None:
+                return
 
         if self._smtp_password is None:
-            self._smtp_password = ui.ui_factory.get_password(
-                'Please enter the SMTP password: %(user)s@%(host)s',
-                user=self._smtp_username,
-                host=self._smtp_server)
+            self._smtp_password = auth.get_password(
+                'smtp', self._smtp_server, self._smtp_username)
 
         self._connection.login(self._smtp_username, self._smtp_password)
 

=== modified file 'bzrlib/tests/HTTPTestUtil.py'
--- bzrlib/tests/HTTPTestUtil.py	2007-07-03 07:03:32 +0000
+++ bzrlib/tests/HTTPTestUtil.py	2007-10-17 15:36:20 +0000
@@ -464,7 +464,7 @@
 
 
 # FIXME: There is some code duplication with
-# _urllib2_wrappers.py.DigestAuthHandler. If that duplciation
+# _urllib2_wrappers.py.DigestAuthHandler. If that duplication
 # grows, it may require a refactoring. Also, we don't implement
 # SHA algorithm nor MD5-sess here, but that does not seem worth
 # it.

=== modified file 'bzrlib/tests/blackbox/test_whoami.py'
--- bzrlib/tests/blackbox/test_whoami.py	2007-06-26 20:32:49 +0000
+++ bzrlib/tests/blackbox/test_whoami.py	2007-11-04 15:24:27 +0000
@@ -45,9 +45,6 @@
         bzr_email = os.environ.get('BZR_EMAIL')
         if bzr_email is not None:
             del os.environ['BZR_EMAIL']
-        bzremail = os.environ.get('BZREMAIL')
-        if bzremail is not None:
-            del os.environ['BZREMAIL']
         try:
             whoami = self.run_bzr("whoami")[0]
             self.assertEquals('Branch Identity <branch@identi.ty>\n', whoami)
@@ -62,16 +59,9 @@
             whoami_email = self.run_bzr("whoami --email")[0]
             self.assertEquals('other@environ.ment\n', whoami_email)
             del os.environ['BZR_EMAIL']
-            os.environ['BZREMAIL'] = 'Yet Another ID <yetother@environ.ment>'
-            whoami, warn = self.run_bzr("whoami")
-            self.assertEquals('Yet Another ID <yetother@environ.ment>\n', whoami)
-            self.assertTrue(len(warn) > 0)
-            del os.environ['BZREMAIL']
         finally:
             if bzr_email is not None:
                 os.environ['BZR_EMAIL'] = bzr_email
-            if bzremail is not None:
-                os.environ['BZREMAIL'] = bzremail
 
     def test_whoami_utf8(self):
         """verify that an identity can be in utf-8."""

=== modified file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	2007-10-09 20:25:21 +0000
+++ bzrlib/tests/test_config.py	2007-10-23 14:31:10 +0000
@@ -24,16 +24,17 @@
 
 #import bzrlib specific imports here
 from bzrlib import (
+    branch,
+    bzrdir,
     config,
     errors,
     osutils,
     mail_client,
+    ui,
     urlutils,
+    tests,
     trace,
     )
-from bzrlib.branch import Branch
-from bzrlib.bzrdir import BzrDir
-from bzrlib.tests import TestCase, TestCaseInTempDir, TestCaseWithTransport
 
 
 sample_long_alias="log -r-15..-1 --line"
@@ -201,7 +202,7 @@
 active = True
 nonactive = False
 """
-class TestConfigObj(TestCase):
+class TestConfigObj(tests.TestCase):
     def test_get_bool(self):
         from bzrlib.config import ConfigObj
         co = ConfigObj(StringIO(bool_config))
@@ -216,7 +217,7 @@
 [section] # line 3
 whocares=notme # line 4
 """
-class TestConfigObjErrors(TestCase):
+class TestConfigObjErrors(tests.TestCase):
 
     def test_duplicate_section_name_error_line(self):
         try:
@@ -226,7 +227,7 @@
         else:
             self.fail('Error in config file not detected')
 
-class TestConfig(TestCase):
+class TestConfig(tests.TestCase):
 
     def test_constructs(self):
         config.Config()
@@ -282,7 +283,7 @@
         self.assertEqual('long', my_config.log_format())
 
 
-class TestConfigPath(TestCase):
+class TestConfigPath(tests.TestCase):
 
     def setUp(self):
         super(TestConfigPath, self).setUp()
@@ -322,7 +323,15 @@
             self.assertEqual(config.locations_config_filename(),
                              '/home/bogus/.bazaar/locations.conf')
 
-class TestIniConfig(TestCase):
+    def test_authentication_config_filename(self):
+        if sys.platform == 'win32':
+            self.assertEqual(config.authentication_config_filename(), 
+                'C:/Documents and Settings/bogus/Application Data/bazaar/2.0/authentication.conf')
+        else:
+            self.assertEqual(config.authentication_config_filename(),
+                             '/home/bogus/.bazaar/authentication.conf')
+
+class TestIniConfig(tests.TestCase):
 
     def test_contructs(self):
         my_config = config.IniBasedConfig("nothing")
@@ -341,7 +350,7 @@
         self.failUnless(my_config._get_parser() is parser)
 
 
-class TestGetConfig(TestCase):
+class TestGetConfig(tests.TestCase):
 
     def test_constructs(self):
         my_config = config.GlobalConfig()
@@ -360,7 +369,7 @@
                                           'utf-8')])
 
 
-class TestBranchConfig(TestCaseWithTransport):
+class TestBranchConfig(tests.TestCaseWithTransport):
 
     def test_constructs(self):
         branch = FakeBranch()
@@ -376,14 +385,14 @@
 
     def test_get_config(self):
         """The Branch.get_config method works properly"""
-        b = BzrDir.create_standalone_workingtree('.').branch
+        b = bzrdir.BzrDir.create_standalone_workingtree('.').branch
         my_config = b.get_config()
         self.assertIs(my_config.get_user_option('wacky'), None)
         my_config.set_user_option('wacky', 'unlikely')
         self.assertEqual(my_config.get_user_option('wacky'), 'unlikely')
 
         # Ensure we get the same thing if we start again
-        b2 = Branch.open('.')
+        b2 = branch.Branch.open('.')
         my_config2 = b2.get_config()
         self.assertEqual(my_config2.get_user_option('wacky'), 'unlikely')
 
@@ -468,7 +477,7 @@
             trace.warning = _warning
 
 
-class TestGlobalConfigItems(TestCase):
+class TestGlobalConfigItems(tests.TestCase):
 
     def test_user_id(self):
         config_file = StringIO(sample_config_text.encode('utf-8'))
@@ -570,7 +579,7 @@
         self.assertEqual(sample_long_alias, my_config.get_alias('ll'))
 
 
-class TestLocationConfig(TestCaseInTempDir):
+class TestLocationConfig(tests.TestCaseInTempDir):
 
     def test_constructs(self):
         my_config = config.LocationConfig('http://example.com')
@@ -922,7 +931,7 @@
 """
 
 
-class TestBranchConfigItems(TestCaseInTempDir):
+class TestBranchConfigItems(tests.TestCaseInTempDir):
 
     def get_branch_config(self, global_config=None, location=None, 
                           location_config=None, branch_data_config=None):
@@ -1072,7 +1081,7 @@
         self.assertRaises(errors.UnknownMailClient, config.get_mail_client)
 
 
-class TestMailAddressExtraction(TestCase):
+class TestMailAddressExtraction(tests.TestCase):
 
     def test_extract_email_address(self):
         self.assertEqual('jane@test.com',
@@ -1081,7 +1090,7 @@
                           config.extract_email_address, 'Jane Tester')
 
 
-class TestTreeConfig(TestCaseWithTransport):
+class TestTreeConfig(tests.TestCaseWithTransport):
 
     def test_get_value(self):
         """Test that retreiving a value from a section is possible"""
@@ -1107,3 +1116,234 @@
         self.assertEqual(value, 'value3-top')
         value = tree_config.get_option('key3', 'SECTION')
         self.assertEqual(value, 'value3-section')
+
+
+class TestAuthenticationConfigFile(tests.TestCase):
+    """Test the authentication.conf file matching"""
+
+    def _got_user_passwd(self, expected_user, expected_password,
+                         config, *args, **kwargs):
+        credentials = config.get_credentials(*args, **kwargs)
+        if credentials is None:
+            user = None
+            password = None
+        else:
+            user = credentials['user']
+            password = credentials['password']
+        self.assertEquals(expected_user, user)
+        self.assertEquals(expected_password, password)
+
+    def  test_empty_config(self):
+        conf = config.AuthenticationConfig(_file=StringIO())
+        self.assertEquals({}, conf._get_config())
+        self._got_user_passwd(None, None, conf, 'http', 'foo.net')
+
+    def test_broken_config(self):
+        conf = config.AuthenticationConfig(_file=StringIO('[DEF'))
+        self.assertRaises(errors.ParseConfigError, conf._get_config)
+
+        conf = config.AuthenticationConfig(_file=StringIO(
+                """[broken]
+scheme=ftp
+user=joe
+verify_certificates=askme # Error: Not a boolean
+"""))
+        self.assertRaises(ValueError, conf.get_credentials, 'ftp', 'foo.net')
+        conf = config.AuthenticationConfig(_file=StringIO(
+                """[broken]
+scheme=ftp
+user=joe
+port=port # Error: Not an int
+"""))
+        self.assertRaises(ValueError, conf.get_credentials, 'ftp', 'foo.net')
+
+    def test_credentials_for_scheme_host(self):
+        conf = config.AuthenticationConfig(_file=StringIO(
+                """# Identity on foo.net
+[ftp definition]
+scheme=ftp
+host=foo.net
+user=joe
+password=secret-pass
+"""))
+        # Basic matching
+        self._got_user_passwd('joe', 'secret-pass', conf, 'ftp', 'foo.net')
+        # different scheme
+        self._got_user_passwd(None, None, conf, 'http', 'foo.net')
+        # different host
+        self._got_user_passwd(None, None, conf, 'ftp', 'bar.net')
+
+    def test_credentials_for_host_port(self):
+        conf = config.AuthenticationConfig(_file=StringIO(
+                """# Identity on foo.net
+[ftp definition]
+scheme=ftp
+port=10021
+host=foo.net
+user=joe
+password=secret-pass
+"""))
+        # No port
+        self._got_user_passwd('joe', 'secret-pass',
+                              conf, 'ftp', 'foo.net', port=10021)
+        # different port
+        self._got_user_passwd(None, None, conf, 'ftp', 'foo.net')
+
+    def test_for_matching_host(self):
+        conf = config.AuthenticationConfig(_file=StringIO(
+                """# Identity on foo.net
+[sourceforge]
+scheme=bzr
+host=bzr.sf.net
+user=joe
+password=joepass
+[sourceforge domain]
+scheme=bzr
+host=.bzr.sf.net
+user=georges
+password=bendover
+"""))
+        # matching domain
+        self._got_user_passwd('georges', 'bendover',
+                              conf, 'bzr', 'foo.bzr.sf.net')
+        # phishing attempt
+        self._got_user_passwd(None, None,
+                              conf, 'bzr', 'bbzr.sf.net')
+
+    def test_for_matching_host_None(self):
+        conf = config.AuthenticationConfig(_file=StringIO(
+                """# Identity on foo.net
+[catchup bzr]
+scheme=bzr
+user=joe
+password=joepass
+[DEFAULT]
+user=georges
+password=bendover
+"""))
+        # match no host
+        self._got_user_passwd('joe', 'joepass',
+                              conf, 'bzr', 'quux.net')
+        # no host but different scheme
+        self._got_user_passwd('georges', 'bendover',
+                              conf, 'ftp', 'quux.net')
+
+    def test_credentials_for_path(self):
+        conf = config.AuthenticationConfig(_file=StringIO(
+                """
+[http dir1]
+scheme=http
+host=bar.org
+path=/dir1
+user=jim
+password=jimpass
+[http dir2]
+scheme=http
+host=bar.org
+path=/dir2
+user=georges
+password=bendover
+"""))
+        # no path no dice
+        self._got_user_passwd(None, None,
+                              conf, 'http', host='bar.org', path='/dir3')
+        # matching path
+        self._got_user_passwd('georges', 'bendover',
+                              conf, 'http', host='bar.org', path='/dir2')
+        # matching subdir
+        self._got_user_passwd('jim', 'jimpass',
+                              conf, 'http', host='bar.org',path='/dir1/subdir')
+
+    def test_credentials_for_user(self):
+        conf = config.AuthenticationConfig(_file=StringIO(
+                """
+[with user]
+scheme=http
+host=bar.org
+user=jim
+password=jimpass
+"""))
+        # Get user
+        self._got_user_passwd('jim', 'jimpass',
+                              conf, 'http', 'bar.org')
+        # Get same user
+        self._got_user_passwd('jim', 'jimpass',
+                              conf, 'http', 'bar.org', user='jim')
+        # Don't get a different user if one is specified
+        self._got_user_passwd(None, None,
+                              conf, 'http', 'bar.org', user='georges')
+
+    def test_verify_certificates(self):
+        conf = config.AuthenticationConfig(_file=StringIO(
+                """
+[self-signed]
+scheme=https
+host=bar.org
+user=jim
+password=jimpass
+verify_certificates=False
+[normal]
+scheme=https
+host=foo.net
+user=georges
+password=bendover
+"""))
+        credentials = conf.get_credentials('https', 'bar.org')
+        self.assertEquals(False, credentials.get('verify_certificates'))
+        credentials = conf.get_credentials('https', 'foo.net')
+        self.assertEquals(True, credentials.get('verify_certificates'))
+
+
+class TestAuthenticationConfig(tests.TestCase):
+    """Test AuthenticationConfig behaviour"""
+
+    def _check_default_prompt(self, expected_prompt_format, scheme,
+                              host=None, port=None, realm=None, path=None):
+        if host is None:
+            host = 'bar.org'
+        user, password = 'jim', 'precious'
+        expected_prompt = expected_prompt_format % {
+            'scheme': scheme, 'host': host, 'port': port,
+            'user': user, 'realm': realm}
+
+        stdout = tests.StringIOWrapper()
+        ui.ui_factory = tests.TestUIFactory(stdin=password + '\n',
+                                            stdout=stdout)
+        # We use an empty conf so that the user is always prompted
+        conf = config.AuthenticationConfig()
+        self.assertEquals(password,
+                          conf.get_password(scheme, host, user, port=port,
+                                            realm=realm, path=path))
+        self.assertEquals(stdout.getvalue(), expected_prompt)
+
+    def test_default_prompts(self):
+        # HTTP prompts can't be tested here, see test_http.py
+        self._check_default_prompt('FTP %(user)s@%(host)s password: ', 'ftp')
+        self._check_default_prompt('FTP %(user)s@%(host)s:%(port)d password: ',
+                                   'ftp', port=10020)
+
+        self._check_default_prompt('SSH %(user)s@%(host)s:%(port)d password: ',
+                                   'ssh', port=12345)
+        # SMTP port handling is a bit special (it's handled if embedded in the
+        # host too)
+        # FIXME: should we: forbid that, extend it to other schemes, leave
+        # things as they are that's fine thank you ?
+        self._check_default_prompt('SMTP %(user)s@%(host)s password: ',
+                                   'smtp')
+        self._check_default_prompt('SMTP %(user)s@%(host)s password: ',
+                                   'smtp', host='bar.org:10025')
+        self._check_default_prompt(
+            'SMTP %(user)s@%(host)s:%(port)d password: ',
+            'smtp', port=10025)
+
+
+# FIXME: Once we have a way to declare authentication to all test servers, we
+# can implement generic tests.
+# test_user_password_in_url
+# test_user_in_url_password_from_config
+# test_user_in_url_password_prompted
+# test_user_in_config
+# test_user_getpass.getuser
+# test_user_prompted ?
+class TestAuthenticationRing(tests.TestCaseWithTransport):
+    pass

=== modified file 'bzrlib/tests/test_ftp_transport.py'
--- bzrlib/tests/test_ftp_transport.py	2007-10-31 09:31:50 +0000
+++ bzrlib/tests/test_ftp_transport.py	2007-11-04 15:29:17 +0000
@@ -14,9 +14,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from cStringIO import StringIO
 import sys
 
 from bzrlib import (
+    config,
     tests,
     transport,
     ui,
@@ -66,38 +68,50 @@
 
 class TestFTPServerUI(TestCaseWithFTPServer):
 
-    def setUp(self):
-        super(TestFTPServerUI, self).setUp()
-        self.old_factory = ui.ui_factory
-        # The following has the unfortunate side-effect of hiding any ouput
-        # during the tests (including pdb prompts). Feel free to comment them
-        # for debugging purposes but leave them in place, there are needed to
-        # run the tests without any console
-        self.old_stdout = sys.stdout
-        sys.stdout = tests.StringIOWrapper()
-        self.addCleanup(self.restoreUIFactory)
-
-    def restoreUIFactory(self):
-        ui.ui_factory = self.old_factory
-        sys.stdout = self.old_stdout
-
-    def test_prompt_for_password(self):
-        t = self.get_transport()
-        # Ensure that the test framework set the password
-        self.assertIsNot(t._password, None)
-        # Reset the password (get_url set the password to 'bar' so we
-        # reset it to None in the transport before the connection).
-        password = t._password
-        t._password = None
-        ui.ui_factory = tests.TestUIFactory(stdin=password+'\n')
-        # Ask the server to check the password
+    def _add_authorized_user(self, user, password):
         server = self.get_server()
         # FIXME: There should be a better way to declare authorized users and
         # passwords to the server
         authorizer = server._ftp_server.authorizer
-        authorizer.secured_user = t._user
+        authorizer.secured_user = user
         authorizer.secured_password = password
+
+    def test_prompt_for_password(self):
+        t = self.get_transport()
+        # Ensure that the test framework set the password
+        self.assertIsNot(t._password, None)
+        # Reset the password (get_url set the password to 'bar' so we
+        # reset it to None in the transport before the connection).
+        password = t._password
+        t._password = None
+        ui.ui_factory = tests.TestUIFactory(stdin=password+'\n',
+                                            stdout=tests.StringIOWrapper())
+        # Ask the server to check the password
+        self._add_authorized_user(t._user, password)
         # Issue a request to the server to connect
         t.has('whatever/not/existing')
         # stdin should be empty (the provided password have been consumed)
         self.assertEqual('', ui.ui_factory.stdin.readline())
+
+    def test_no_prompt_for_password_when_using_auth_config(self):
+        t = self.get_transport()
+        # Reset the password (get_url set the password to 'bar' so we
+        # reset it to None in the transport before the connection).
+        password = t._password
+        t._password = None
+        ui.ui_factory = tests.TestUIFactory(stdin='precious\n',
+                                            stdout=tests.StringIOWrapper())
+        # Ask the server to check the password
+        self._add_authorized_user(t._user, password)
+
+        # Create a config file with the right password
+        conf = config.AuthenticationConfig()
+        conf._get_config().update({'ftptest': {'scheme': 'ftp',
+                                               'user': t._user,
+                                               'password': password}})
+        conf._save()
+        # Issue a request to the server to connect
+        t.put_bytes('foo', 'test bytes\n')
+        self.assertEqual('test bytes\n', t.get_bytes('foo'))
+        # stdin should have  been left untouched
+        self.assertEqual('precious\n', ui.ui_factory.stdin.readline())

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2007-11-02 05:12:17 +0000
+++ bzrlib/tests/test_http.py	2007-11-04 15:29:17 +0000
@@ -29,6 +29,7 @@
 
 import bzrlib
 from bzrlib import (
+    config,
     errors,
     osutils,
     ui,
@@ -78,7 +79,6 @@
     )
 from bzrlib.transport.http._urllib import HttpTransport_urllib
 from bzrlib.transport.http._urllib2_wrappers import (
-    PasswordManager,
     ProxyHandler,
     Request,
     )
@@ -204,7 +204,7 @@
     def test_invalid_http_urls(self):
         """Trap invalid construction of urls"""
         t = self._transport('http://bazaar-vcs.org/bzr/bzr.dev/')
-        self.assertRaises((errors.InvalidURL, errors.ConnectionError),
+        self.assertRaises(errors.InvalidURL,
                           self._transport,
                           'http://http://bazaar-vcs.org/bzr/bzr.dev/')
 
@@ -797,7 +797,7 @@
             osutils.set_or_unset_env(name, value)
 
     def _proxied_request(self):
-        handler = ProxyHandler(PasswordManager())
+        handler = ProxyHandler()
         request = Request('GET','http://baz/buzzle')
         handler.set_proxy(request, 'http')
         return request
@@ -832,10 +832,6 @@
     # FIXME: We don't have an https server available, so we don't
     # test https connections.
 
-    # FIXME: Once the test suite is better fitted to test
-    # authorization schemes, test proxy authorizations too (see
-    # bug #83954).
-
     def setUp(self):
         TestCaseWithTwoWebservers.setUp(self)
         self.build_tree_contents([('foo', 'contents of foo\n'),
@@ -1210,6 +1206,8 @@
     either TestCaseWithWebserver or TestCaseWithTwoWebservers.
     """
 
+    _password_prompt_prefix = ''
+
     def setUp(self):
         """Set up the test environment
 
@@ -1221,18 +1219,6 @@
         """
         self.build_tree_contents([('a', 'contents of a\n'),
                                   ('b', 'contents of b\n'),])
-        self.old_factory = ui.ui_factory
-        # The following has the unfortunate side-effect of hiding any ouput
-        # during the tests (including pdb prompts). Feel free to comment them
-        # for debugging purposes but leave them in place, there are needed to
-        # run the tests without any console
-        self.old_stdout = sys.stdout
-        sys.stdout = StringIOWrapper()
-        self.addCleanup(self.restoreUIFactory)
-
-    def restoreUIFactory(self):
-        ui.ui_factory = self.old_factory
-        sys.stdout = self.old_stdout
 
     def get_user_url(self, user=None, password=None):
         """Build an url embedding user and password"""
@@ -1286,10 +1272,13 @@
     def test_prompt_for_password(self):
         self.server.add_user('joe', 'foo')
         t = self.get_user_transport('joe', None)
-        ui.ui_factory = TestUIFactory(stdin='foo\n')
+        stdout = StringIOWrapper()
+        ui.ui_factory = TestUIFactory(stdin='foo\n', stdout=stdout)
         self.assertEqual('contents of a\n',t.get('a').read())
         # stdin should be empty
         self.assertEqual('', ui.ui_factory.stdin.readline())
+        self._check_password_prompt(t._unqualified_scheme, 'joe',
+                                    stdout.getvalue())
         # And we shouldn't prompt again for a different request
         # against the same transport.
         self.assertEqual('contents of b\n',t.get('b').read())
@@ -1299,6 +1288,36 @@
         # Only one 'Authentication Required' error should occur
         self.assertEqual(1, self.server.auth_required_errors)
 
+    def _check_password_prompt(self, scheme, user, actual_prompt):
+        expected_prompt = (self._password_prompt_prefix
+                           + ("%s %s@%s:%d, Realm: '%s' password: "
+                              % (scheme.upper(),
+                                 user, self.server.host, self.server.port,
+                                 self.server.auth_realm)))
+        self.assertEquals(expected_prompt, actual_prompt)
+
+    def test_no_prompt_for_password_when_using_auth_config(self):
+        user =' joe'
+        password = 'foo'
+        stdin_content = 'bar\n'  # Not the right password
+        self.server.add_user(user, password)
+        t = self.get_user_transport(user, None)
+        ui.ui_factory = TestUIFactory(stdin=stdin_content,
+                                      stdout=StringIOWrapper())
+        # Create a minimal config file with the right password
+        conf = config.AuthenticationConfig()
+        conf._get_config().update(
+            {'httptest': {'scheme': 'http', 'port': self.server.port,
+                          'user': user, 'password': password}})
+        conf._save()
+        # Issue a request to the server to connect
+        self.assertEqual('contents of a\n',t.get('a').read())
+        # stdin should have  been left untouched
+        self.assertEqual(stdin_content, ui.ui_factory.stdin.readline())
+        # Only one 'Authentication Required' error should occur
+        self.assertEqual(1, self.server.auth_required_errors)
+
+
 
 class TestHTTPAuth(TestAuth):
     """Test HTTP authentication schemes.
@@ -1323,6 +1342,8 @@
     Daughter classes MUST also inherit from TestCaseWithWebserver.
     """
     _auth_header = 'Proxy-authorization'
+    _password_prompt_prefix = 'Proxy '
+
 
     def setUp(self):
         TestCaseWithWebserver.setUp(self)

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2007-10-25 08:24:09 +0000
+++ bzrlib/tests/test_smart_transport.py	2007-11-04 15:29:17 +0000
@@ -2058,8 +2058,10 @@
 
         http_transport = self.get_readonly_transport()
         medium = http_transport.get_smart_medium()
-        #remote_transport = RemoteTransport('fake_url', medium)
-        remote_transport = remote.RemoteTransport('/', medium=medium)
+        # Since we provide the medium, the url below will be mostly ignored
+        # during the test, as long as the path is '/'.
+        remote_transport = remote.RemoteTransport('bzr://fake_host/',
+                                                  medium=medium)
         self.assertEqual(
             [(0, "c")], list(remote_transport.readv("data-file", [(0,1)])))
 

=== modified file 'bzrlib/tests/test_smtp_connection.py'
--- bzrlib/tests/test_smtp_connection.py	2007-10-10 03:43:04 +0000
+++ bzrlib/tests/test_smtp_connection.py	2007-10-23 07:15:13 +0000
@@ -19,15 +19,16 @@
 import errno
 import smtplib
 import socket
+import sys
 
 from bzrlib import (
     config,
+    email_message,
     errors,
+    smtp_connection,
+    tests,
+    ui,
     )
-from bzrlib.email_message import EmailMessage
-from bzrlib.errors import NoDestinationAddress
-from bzrlib.tests import TestCase
-from bzrlib.smtp_connection import SMTPConnection
 
 
 def connection_refuser():
@@ -81,13 +82,21 @@
             return 200, 'starttls success'
 
 
-class TestSMTPConnection(TestCase):
+class WideOpenSMTPFactory(StubSMTPFactory):
+    """A fake smtp server that implements login by accepting anybody."""
+
+    def login(self, user, password):
+        pass
+
+
+class TestSMTPConnection(tests.TestCaseInTempDir):
 
     def get_connection(self, text, smtp_factory=None):
         my_config = config.GlobalConfig()
         config_file = StringIO(text)
         my_config._get_parser(config_file)
-        return SMTPConnection(my_config, _smtp_factory=smtp_factory)
+        return smtp_connection.SMTPConnection(my_config,
+                                              _smtp_factory=smtp_factory)
 
     def test_defaults(self):
         conn = self.get_connection('')
@@ -113,13 +122,46 @@
         conn = self.get_connection('[DEFAULT]\nsmtp_username=joebody\n')
         self.assertEqual(u'joebody', conn._smtp_username)
 
-    def test_smtp_password(self):
+    def test_smtp_password_from_config(self):
         conn = self.get_connection('')
         self.assertIs(None, conn._smtp_password)
 
         conn = self.get_connection('[DEFAULT]\nsmtp_password=mypass\n')
         self.assertEqual(u'mypass', conn._smtp_password)
 
+    def test_smtp_password_from_user(self):
+        user = 'joe'
+        password = 'hispass'
+        factory = WideOpenSMTPFactory()
+        conn = self.get_connection('[DEFAULT]\nsmtp_username=%s\n' % user,
+                                   smtp_factory=factory)
+        self.assertIs(None, conn._smtp_password)
+
+        ui.ui_factory = tests.TestUIFactory(stdin=password + '\n',
+                                            stdout=tests.StringIOWrapper())
+        conn._connect()
+        self.assertEqual(password, conn._smtp_password)
+        # stdin should be empty (the provided password have been consumed)
+        self.assertEqual('', ui.ui_factory.stdin.readline())
+
+    def test_smtp_password_from_auth_config(self):
+        user = 'joe'
+        password = 'hispass'
+        factory = WideOpenSMTPFactory()
+        conn = self.get_connection('[DEFAULT]\nsmtp_username=%s\n' % user,
+                                   smtp_factory=factory)
+        self.assertEqual(user, conn._smtp_username)
+        self.assertIs(None, conn._smtp_password)
+        # Create a config file with the right password
+        conf = config.AuthenticationConfig()
+        conf._get_config().update({'smtptest':
+                                       {'scheme': 'smtp', 'user':user,
+                                        'password': password}})
+        conf._save()
+
+        conn._connect()
+        self.assertEqual(password, conn._smtp_password)
+
     def test_create_connection(self):
         factory = StubSMTPFactory()
         conn = self.get_connection('', smtp_factory=factory)
@@ -174,7 +216,7 @@
     def test_get_message_addresses(self):
         msg = Message()
 
-        from_, to = SMTPConnection.get_message_addresses(msg)
+        from_, to = smtp_connection.SMTPConnection.get_message_addresses(msg)
         self.assertEqual('', from_)
         self.assertEqual([], to)
 
@@ -183,18 +225,19 @@
         msg['CC'] = u'Pepe P\xe9rez <pperez@ejemplo.com>'
         msg['Bcc'] = 'user@localhost'
 
-        from_, to = SMTPConnection.get_message_addresses(msg)
+        from_, to = smtp_connection.SMTPConnection.get_message_addresses(msg)
         self.assertEqual('jrandom@example.com', from_)
         self.assertEqual(sorted(['john@doe.com', 'jane@doe.com',
             'pperez@ejemplo.com', 'user@localhost']), sorted(to))
 
         # now with bzrlib's EmailMessage
-        msg = EmailMessage('"J. Random Developer" <jrandom@example.com>', [
-            'John Doe <john@doe.com>', 'Jane Doe <jane@doe.com>',
-            u'Pepe P\xe9rez <pperez@ejemplo.com>', 'user@localhost' ],
+        msg = email_message.EmailMessage(
+            '"J. Random Developer" <jrandom@example.com>',
+            ['John Doe <john@doe.com>', 'Jane Doe <jane@doe.com>',
+             u'Pepe P\xe9rez <pperez@ejemplo.com>', 'user@localhost' ],
             'subject')
 
-        from_, to = SMTPConnection.get_message_addresses(msg)
+        from_, to = smtp_connection.SMTPConnection.get_message_addresses(msg)
         self.assertEqual('jrandom@example.com', from_)
         self.assertEqual(sorted(['john@doe.com', 'jane@doe.com',
             'pperez@ejemplo.com', 'user@localhost']), sorted(to))
@@ -206,13 +249,16 @@
 
         msg = Message()
         msg['From'] = '"J. Random Developer" <jrandom@example.com>'
-        self.assertRaises(NoDestinationAddress,
-                SMTPConnection(FakeConfig()).send_email, msg)
-
-        msg = EmailMessage('from@from.com', '', 'subject')
-        self.assertRaises(NoDestinationAddress,
-                SMTPConnection(FakeConfig()).send_email, msg)
-
-        msg = EmailMessage('from@from.com', [], 'subject')
-        self.assertRaises(NoDestinationAddress,
-                SMTPConnection(FakeConfig()).send_email, msg)
+        self.assertRaises(
+            errors.NoDestinationAddress,
+            smtp_connection.SMTPConnection(FakeConfig()).send_email, msg)
+
+        msg = email_message.EmailMessage('from@from.com', '', 'subject')
+        self.assertRaises(
+            errors.NoDestinationAddress,
+            smtp_connection.SMTPConnection(FakeConfig()).send_email, msg)
+
+        msg = email_message.EmailMessage('from@from.com', [], 'subject')
+        self.assertRaises(
+            errors.NoDestinationAddress,
+            smtp_connection.SMTPConnection(FakeConfig()).send_email, msg)

=== modified file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	2007-10-08 02:22:17 +0000
+++ bzrlib/tests/test_transport.py	2007-10-22 21:19:13 +0000
@@ -663,11 +663,13 @@
         self.assertEquals(t.relpath('sftp://host.com/dev/%path/sub'), 'sub')
 
     def test_connection_sharing_propagate_credentials(self):
-        t = ConnectedTransport('foo://user@host.com/abs/path')
+        t = ConnectedTransport('ftp://user@host.com/abs/path')
+        self.assertEquals('user', t._user)
+        self.assertEquals('host.com', t._host)
         self.assertIs(None, t._get_connection())
         self.assertIs(None, t._password)
         c = t.clone('subdir')
-        self.assertEquals(None, c._get_connection())
+        self.assertIs(None, c._get_connection())
         self.assertIs(None, t._password)
 
         # Simulate the user entering a password

=== modified file 'bzrlib/tests/test_transport_implementations.py'
--- bzrlib/tests/test_transport_implementations.py	2007-10-08 04:53:50 +0000
+++ bzrlib/tests/test_transport_implementations.py	2007-11-04 15:29:17 +0000
@@ -1110,7 +1110,7 @@
 
         def new_url(scheme=None, user=None, password=None,
                     host=None, port=None, path=None):
-            """Build a new url from t.base chaging only parts of it.
+            """Build a new url from t.base changing only parts of it.
 
             Only the parameters different from None will be changed.
             """
@@ -1123,7 +1123,11 @@
             if path     is None: path     = t._path
             return t._unsplit_url(scheme, user, password, host, port, path)
 
-        self.assertIsNot(t, t._reuse_for(new_url(scheme='foo')))
+        if t._scheme == 'ftp':
+            scheme = 'sftp'
+        else:
+            scheme = 'ftp'
+        self.assertIsNot(t, t._reuse_for(new_url(scheme=scheme)))
         if t._user == 'me':
             user = 'you'
         else:

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2007-10-24 18:19:51 +0000
+++ bzrlib/transport/__init__.py	2007-11-04 15:29:17 +0000
@@ -1356,6 +1356,9 @@
             except ValueError:
                 raise errors.InvalidURL('invalid port number %s in url:\n%s' %
                                         (port, url))
+        if host == '':
+            raise errors.InvalidURL('Host empty in: %s' % url)
+
         host = urllib.unquote(host)
         path = urllib.unquote(path)
 

=== modified file 'bzrlib/transport/ftp.py'
--- bzrlib/transport/ftp.py	2007-10-31 09:31:50 +0000
+++ bzrlib/transport/ftp.py	2007-11-04 15:29:17 +0000
@@ -36,6 +36,7 @@
 from warnings import warn
 
 from bzrlib import (
+    config,
     errors,
     osutils,
     urlutils,
@@ -43,14 +44,16 @@
 from bzrlib.trace import mutter, warning
 from bzrlib.transport import (
     AppendBasedFileStream,
+    ConnectedTransport,
     _file_streams,
+    register_urlparse_netloc_protocol,
     Server,
-    ConnectedTransport,
     )
 from bzrlib.transport.local import LocalURLServer
 import bzrlib.ui
 
-_have_medusa = False
+
+register_urlparse_netloc_protocol('aftp')
 
 
 class FtpPathError(errors.PathError):
@@ -115,27 +118,33 @@
         in base url at transport creation time.
         """
         if credentials is None:
-            password = self._password
+            user, password = self._user, self._password
         else:
-            password = credentials
+            user, password = credentials
+
+        auth = config.AuthenticationConfig()
+        if user is None:
+            user = auth.get_user('ftp', self._host, port=self._port)
+            if user is None:
+                # Default to local user
+                user = getpass.getuser()
 
         mutter("Constructing FTP instance against %r" %
-               ((self._host, self._port, self._user, '********',
+               ((self._host, self._port, user, '********',
                 self.is_active),))
         try:
             connection = ftplib.FTP()
             connection.connect(host=self._host, port=self._port)
-            if self._user and self._user != 'anonymous' and \
+            if user and user != 'anonymous' and \
                     password is None: # '' is a valid password
-                get_password = bzrlib.ui.ui_factory.get_password
-                password = get_password(prompt='FTP %(user)s@%(host)s password',
-                                        user=self._user, host=self._host)
-            connection.login(user=self._user, passwd=password)
+                password = auth.get_password('ftp', self._host, user,
+                                             port=self._port)
+            connection.login(user=user, passwd=password)
             connection.set_pasv(not self.is_active)
         except ftplib.error_perm, e:
             raise errors.TransportError(msg="Error setting up connection:"
                                         " %s" % str(e), orig_error=e)
-        return connection, password
+        return connection, (user, password)
 
     def _reconnect(self):
         """Create a new connection with the previously used credentials"""

=== modified file 'bzrlib/transport/http/_urllib.py'
--- bzrlib/transport/http/_urllib.py	2007-10-31 12:38:11 +0000
+++ bzrlib/transport/http/_urllib.py	2007-11-04 15:44:32 +0000
@@ -29,8 +29,6 @@
 from bzrlib.transport.http._urllib2_wrappers import (
     Opener,
     Request,
-    extract_authentication_uri,
-    extract_credentials,
     )
 
 
@@ -57,6 +55,8 @@
         path = self._combine_paths(self._path, relative)
         # urllib2 will be confused if it find authentication
         # info (user, password) in the urls. So we handle them separatly.
+
+        # rhaaaa ! confused where ? confused when ? --vila 20070922
         return self._unsplit_url(self._unqualified_scheme,
                                  None, None, self._host, self._port, path)
 
@@ -71,18 +71,16 @@
             request.connection = connection
             (auth, proxy_auth) = self._get_credentials()
         else:
-            # First request, intialize credentials
+            # First request, intialize credentials.
+            # scheme and realm will be set by the _urllib2_wrappers.AuthHandler
             user = self._user
             password = self._password
-            authuri = self._remote_path('.')
-            auth = {'user': user, 'password': password, 'authuri': authuri}
-
-            if user and password is not None: # '' is a valid password
-                # Make the (user, password) available to urllib2
-                # We default to a realm of None to catch them all.
-                self._opener.password_manager.add_password(None, authuri,
-                                                           user, password)
-            proxy_auth = {}
+            auth = dict(host=self._host, port=self._port,
+                        user=user, password=password,
+                        protocol=self._unqualified_scheme,
+                        path=self._path)
+            # Proxy initialization will be done by first proxied request
+            proxy_auth = dict()
         # Ensure authentication info is provided
         request.auth = auth
         request.proxy_auth = proxy_auth

=== modified file 'bzrlib/transport/http/_urllib2_wrappers.py'
--- bzrlib/transport/http/_urllib2_wrappers.py	2007-07-04 12:28:56 +0000
+++ bzrlib/transport/http/_urllib2_wrappers.py	2007-10-23 10:58:09 +0000
@@ -34,11 +34,6 @@
 
 DEBUG = 0
 
-# TODO: It may be possible to share the password_manager across
-# all transports by prefixing the realm by the protocol used
-# (especially if other protocols do not use realms). See
-# PasswordManager below.
-
 # FIXME: Oversimplifying, two kind of exceptions should be
 # raised, once a request is issued: URLError before we have been
 # able to process the response, HTTPError after that. Process the
@@ -60,12 +55,13 @@
 
 from bzrlib import __version__ as bzrlib_version
 from bzrlib import (
+    config,
     errors,
+    transport,
     ui,
     )
 
 
-
 # We define our own Response class to keep our httplib pipe clean
 class Response(httplib.HTTPResponse):
     """Custom HTTPResponse, to avoid the need to decorate.
@@ -193,7 +189,7 @@
         # Unless told otherwise, redirections are not followed
         self.follow_redirections = False
         # auth and proxy_auth are dicts containing, at least
-        # (scheme, url, realm, user, password).
+        # (scheme, host, port, realm, user, password, protocol, path).
         # The dict entries are mostly handled by the AuthHandler.
         # Some authentication schemes may add more entries.
         self.auth = {}
@@ -240,54 +236,6 @@
         urllib2.Request.set_proxy(self, proxy, type)
 
 
-def extract_credentials(url):
-    """Extracts credentials information from url.
-
-    Get user and password from url of the form: http://user:pass@host/path
-    :returns: (clean_url, user, password)
-    """
-    scheme, netloc, path, query, fragment = urlparse.urlsplit(url)
-
-    if '@' in netloc:
-        auth, netloc = netloc.split('@', 1)
-        if ':' in auth:
-            user, password = auth.split(':', 1)
-        else:
-            user, password = auth, None
-        user = urllib.unquote(user)
-        if password is not None:
-            password = urllib.unquote(password)
-    else:
-        user = None
-        password = None
-
-    # Build the clean url
-    clean_url = urlparse.urlunsplit((scheme, netloc, path, query, fragment))
-
-    return clean_url, user, password
-
-def extract_authentication_uri(url):
-    """Extract the authentication uri from any url.
-
-    In the context of bzr, we simplified the authentication uri
-    to the host only. For the transport lifetime, we allow only
-    one user by realm on a given host. I.e. handling several
-    users for different paths for the same realm should be done
-    at a higher level.
-    """
-    scheme, host, path, query, fragment = urlparse.urlsplit(url)
-    return '%s://%s' % (scheme, host)
-
-
-# The AuthHandler classes handle the authentication of the requests, to do
-# that, they need a PasswordManager *at build time*. We also need one to reuse
-# the passwords entered by the user.
-class PasswordManager(urllib2.HTTPPasswordMgrWithDefaultRealm):
-
-    def __init__(self):
-        urllib2.HTTPPasswordMgrWithDefaultRealm.__init__(self)
-
-
 class ConnectionHandler(urllib2.BaseHandler):
     """Provides connection-sharing by pre-processing requests.
 
@@ -737,9 +685,8 @@
     handler_order = 100
     _debuglevel = DEBUG
 
-    def __init__(self, password_manager, proxies=None):
+    def __init__(self, proxies=None):
         urllib2.ProxyHandler.__init__(self, proxies)
-        self.password_manager = password_manager
         # First, let's get rid of urllib2 implementation
         for type, proxy in self.proxies.items():
             if self._debuglevel > 0:
@@ -782,7 +729,7 @@
 
     def proxy_bypass(self, host):
         """Check if host should be proxied or not"""
-        no_proxy = self.get_proxy_env_var('no', None)
+        no_proxy = self.get_proxy_env_var('no', default_to=None)
         if no_proxy is None:
             return False
         hhost, hport = urllib.splitport(host)
@@ -814,33 +761,26 @@
         # grok user:password@host:port as well as
         # http://user:password@host:port
 
-        # Extract credentials from the url and store them in the
-        # password manager so that the proxy AuthHandler can use
-        # them later.
-        proxy, user, password = extract_credentials(proxy)
+        (scheme, user, password,
+         host, port, path) = transport.ConnectedTransport._split_url(proxy)
+
         if request.proxy_auth == {}:
-            # No proxy auth parameter are available, we are
-            # handling the first proxied request, intialize.
-            # scheme and realm will be set by the AuthHandler
-            authuri = extract_authentication_uri(proxy)
-            request.proxy_auth = {'user': user, 'password': password,
-                                  'authuri': authuri}
-            if user and password is not None: # '' is a valid password
-                # We default to a realm of None to catch them all.
-                self.password_manager.add_password(None, authuri,
-                                                   user, password)
-        orig_type = request.get_type()
-        scheme, r_scheme = urllib.splittype(proxy)
-        if self._debuglevel > 0:
-            print 'scheme: %s, r_scheme: %s' % (scheme, r_scheme)
-        host, XXX = urllib.splithost(r_scheme)
-        if host is None:
-            raise errors.InvalidURL(proxy,
-                                    'Invalid syntax in proxy env variable')
-        host = urllib.unquote(host)
-        request.set_proxy(host, type)
-        if self._debuglevel > 0:
-            print 'set_proxy: proxy set to %s://%s' % (type, host)
+            # No proxy auth parameter are available, we are handling the first
+            # proxied request, intialize.  scheme (the authentication scheme)
+            # and realm will be set by the AuthHandler
+            request.proxy_auth = {
+                                  'host': host, 'port': port,
+                                  'user': user, 'password': password,
+                                  'protocol': scheme,
+                                   # We ignore path since we connect to a proxy
+                                  'path': None}
+        if port is None:
+            phost = host
+        else:
+            phost = host + ':%d' % port
+        request.set_proxy(phost, type)
+        if self._debuglevel > 0:
+            print 'set_proxy: proxy set to %s://%s' % (type, phost)
         return request
 
 
@@ -882,7 +822,7 @@
       successful and the request authentication parameters have been updated.
     """
 
-    _max_retry = 2
+    _max_retry = 3
     """We don't want to retry authenticating endlessly"""
 
     # The following attributes should be defined by daughter
@@ -890,10 +830,10 @@
     # - auth_required_header:  the header received from the server
     # - auth_header: the header sent in the request
 
-    def __init__(self, password_manager):
-        self.password_manager = password_manager
-        self.find_user_password = password_manager.find_user_password
-        self.add_password = password_manager.add_password
+    def __init__(self):
+        # We want to know when we enter into an try/fail cycle of
+        # authentications so we initialize to None to indicate that we aren't
+        # in such a cycle by default.
         self._retry_count = None
 
     def update_auth(self, auth, key, value):
@@ -912,8 +852,8 @@
         """
         # Don't try  to authenticate endlessly
         if self._retry_count is None:
-            # The retry being recusrsive calls, None identify the first try
-            self._retry_count = 0
+            # The retry being recusrsive calls, None identify the first retry
+            self._retry_count = 1
         else:
             self._retry_count += 1
             if self._retry_count > self._max_retry:
@@ -935,8 +875,8 @@
         if self.auth_match(server_header, auth):
             # auth_match may have modified auth (by adding the
             # password or changing the realm, for example)
-            if request.get_header(self.auth_header, None) is not None \
-                    and not auth['modified']:
+            if (request.get_header(self.auth_header, None) is not None
+                and not auth['modified']):
                 # We already tried that, give up
                 return None
 
@@ -998,26 +938,46 @@
         # It may happen that we need to reconnect later, let's be ready
         self._retry_count = None
 
-    def get_password(self, user, authuri, realm=None):
+    def get_user_password(self, auth):
         """Ask user for a password if none is already available."""
-        user_found, password = self.find_user_password(realm, authuri)
-        if user_found != user:
-            # FIXME: write a test for that case
-            password = None
+        auth_conf = config.AuthenticationConfig()
+        user = auth['user']
+        password = auth['password']
+        realm = auth['realm']
+
+        if user is None:
+            user = auth.get_user(auth['protocol'], auth['host'],
+                                 port=auth['port'], path=auth['path'],
+                                 realm=realm)
+            if user is None:
+                # Default to local user
+                user = getpass.getuser()
 
         if password is None:
-            # Prompt user only if we can't find a password
-            if realm:
-                realm_prompt = " Realm: '%s'" % realm
-            else:
-                realm_prompt = ''
-            scheme, host, path, query, fragment = urlparse.urlsplit(authuri)
-            password = ui.ui_factory.get_password(prompt=self.password_prompt,
-                                                  user=user, host=host,
-                                                  realm=realm_prompt)
-            if password is not None:
-                self.add_password(realm, authuri, user, password)
-        return password
+            password = auth_conf.get_password(
+                auth['protocol'], auth['host'], user, port=auth['port'],
+                path=auth['path'], realm=realm,
+                prompt=self.build_password_prompt(auth))
+
+        return user, password
+
+    def _build_password_prompt(self, auth):
+        """Build a prompt taking the protocol used into account.
+
+        The AuthHandler is used by http and https, we want that information in
+        the prompt, so we build the prompt from the authentication dict which
+        contains all the needed parts.
+
+        Also, hhtp and proxy AuthHandlers present different prompts to the
+        user. The daughter classes hosuld implements a public
+        build_password_prompt using this method.
+        """
+        prompt = '%s' % auth['protocol'].upper() + ' %(user)s@%(host)s'
+        realm = auth['realm']
+        if realm is not None:
+            prompt += ", Realm: '%s'" % realm
+        prompt += ' password'
+        return prompt
 
     def http_request(self, request):
         """Insert an authentication header if information is available"""
@@ -1056,9 +1016,9 @@
             # Put useful info into auth
             self.update_auth(auth, 'scheme', scheme)
             self.update_auth(auth, 'realm', realm)
-            if auth.get('password',None) is None:
-                password = self.get_password(auth['user'], auth['authuri'],
-                                             auth['realm'])
+            if auth['user'] is None or auth['password'] is None:
+                user, password = self.get_user_password(auth)
+                self.update_auth(auth, 'user', user)
                 self.update_auth(auth, 'password', password)
         return match is not None
 
@@ -1118,16 +1078,17 @@
             return False
 
         realm = req_auth.get('realm', None)
-        if auth.get('password',None) is None:
-            auth['password'] = self.get_password(auth['user'],
-                                                 auth['authuri'],
-                                                 realm)
         # Put useful info into auth
+        self.update_auth(auth, 'scheme', scheme)
+        self.update_auth(auth, 'realm', realm)
+        if auth['user'] is None or auth['password'] is None:
+            user, password = self.get_user_password(auth)
+            self.update_auth(auth, 'user', user)
+            self.update_auth(auth, 'password', password)
+
         try:
-            self.update_auth(auth, 'scheme', scheme)
             if req_auth.get('algorithm', None) is not None:
                 self.update_auth(auth, 'algorithm', req_auth.get('algorithm'))
-            self.update_auth(auth, 'realm', realm)
             nonce = req_auth['nonce']
             if auth.get('nonce', None) != nonce:
                 # A new nonce, never used
@@ -1188,7 +1149,6 @@
     the auth request attribute.
     """
 
-    password_prompt = 'HTTP %(user)s@%(host)s%(realm)s password'
     auth_required_header = 'www-authenticate'
     auth_header = 'Authorization'
 
@@ -1200,6 +1160,9 @@
         """Set the auth params for the request"""
         request.auth = auth
 
+    def build_password_prompt(self, auth):
+        return self._build_password_prompt(auth)
+
     def http_error_401(self, req, fp, code, msg, headers):
         return self.auth_required(req, headers)
 
@@ -1212,7 +1175,6 @@
     the proxy_auth request attribute..
     """
 
-    password_prompt = 'Proxy %(user)s@%(host)s%(realm)s password'
     auth_required_header = 'proxy-authenticate'
     # FIXME: the correct capitalization is Proxy-Authorization,
     # but python-2.4 urllib2.Request insist on using capitalize()
@@ -1227,6 +1189,11 @@
         """Set the auth params for the request"""
         request.proxy_auth = auth
 
+    def build_password_prompt(self, auth):
+        prompt = self._build_password_prompt(auth)
+        prompt = 'Proxy ' + prompt
+        return prompt
+
     def http_error_407(self, req, fp, code, msg, headers):
         return self.auth_required(req, headers)
 
@@ -1307,14 +1274,13 @@
                  connection=ConnectionHandler,
                  redirect=HTTPRedirectHandler,
                  error=HTTPErrorProcessor,):
-        self.password_manager = PasswordManager()
         self._opener = urllib2.build_opener( \
             connection, redirect, error,
-            ProxyHandler(self.password_manager),
-            HTTPBasicAuthHandler(self.password_manager),
-            HTTPDigestAuthHandler(self.password_manager),
-            ProxyBasicAuthHandler(self.password_manager),
-            ProxyDigestAuthHandler(self.password_manager),
+            ProxyHandler(),
+            HTTPBasicAuthHandler(),
+            HTTPDigestAuthHandler(),
+            ProxyBasicAuthHandler(),
+            ProxyDigestAuthHandler(),
             HTTPHandler,
             HTTPSHandler,
             HTTPDefaultErrorHandler,

=== modified file 'bzrlib/transport/ssh.py'
--- bzrlib/transport/ssh.py	2007-08-31 05:38:10 +0000
+++ bzrlib/transport/ssh.py	2007-10-23 07:25:41 +0000
@@ -24,18 +24,13 @@
 import subprocess
 import sys
 
-from bzrlib.config import config_dir, ensure_config_dir_exists
-from bzrlib.errors import (ConnectionError,
-                           ParamikoNotPresent,
-                           SocketConnectionError,
-                           SSHVendorNotFound,
-                           TransportError,
-                           UnknownSSH,
-                           )
-
-from bzrlib.osutils import pathjoin
-from bzrlib.trace import mutter, warning
-import bzrlib.ui
+from bzrlib import (
+    config,
+    errors,
+    osutils,
+    trace,
+    ui,
+    )
 
 try:
     import paramiko
@@ -98,7 +93,7 @@
             try:
                 vendor = self._ssh_vendors[vendor_name]
             except KeyError:
-                raise UnknownSSH(vendor_name)
+                raise errors.UnknownSSH(vendor_name)
             return vendor
         return None
 
@@ -122,16 +117,16 @@
         """
         vendor = None
         if 'OpenSSH' in version:
-            mutter('ssh implementation is OpenSSH')
+            trace.mutter('ssh implementation is OpenSSH')
             vendor = OpenSSHSubprocessVendor()
         elif 'SSH Secure Shell' in version:
-            mutter('ssh implementation is SSH Corp.')
+            trace.mutter('ssh implementation is SSH Corp.')
             vendor = SSHCorpSubprocessVendor()
         elif 'plink' in version and args[0] == 'plink':
-            # Checking if "plink" was the executed argument as Windows sometimes 
-            # reports 'ssh -V' incorrectly with 'plink' in it's version. 
-            # See https://bugs.launchpad.net/bzr/+bug/107155
-            mutter("ssh implementation is Putty's plink.")
+            # Checking if "plink" was the executed argument as Windows
+            # sometimes reports 'ssh -V' incorrectly with 'plink' in it's
+            # version.  See https://bugs.launchpad.net/bzr/+bug/107155
+            trace.mutter("ssh implementation is Putty's plink.")
             vendor = PLinkSubprocessVendor()
         return vendor
 
@@ -156,10 +151,10 @@
             if vendor is None:
                 vendor = self._get_vendor_by_inspection()
                 if vendor is None:
-                    mutter('falling back to default implementation')
+                    trace.mutter('falling back to default implementation')
                     vendor = self._default_ssh_vendor
                     if vendor is None:
-                        raise SSHVendorNotFound()
+                        raise errors.SSHVendorNotFound()
             self._cached_ssh_vendor = vendor
         return self._cached_ssh_vendor
 
@@ -182,7 +177,7 @@
 
     def __init__(self, sock):
         self.__socket = sock
- 
+
     def send(self, data):
         return self.__socket.send(data)
 
@@ -198,7 +193,7 @@
 
 class SSHVendor(object):
     """Abstract base class for SSH vendor implementations."""
-    
+
     def connect_sftp(self, username, password, host, port):
         """Make an SSH connection, and return an SFTPClient.
         
@@ -221,7 +216,7 @@
             method that returns a pair of (read, write) filelike objects.
         """
         raise NotImplementedError(self.connect_ssh)
-        
+
     def _raise_connection_error(self, host, port=None, orig_error=None,
                                 msg='Unable to connect to SSH host'):
         """Raise a SocketConnectionError with properly formatted host.
@@ -229,13 +224,13 @@
         This just unifies all the locations that try to raise ConnectionError,
         so that they format things properly.
         """
-        raise SocketConnectionError(host=host, port=port, msg=msg,
-                                    orig_error=orig_error)
+        raise errors.SocketConnectionError(host=host, port=port, msg=msg,
+                                           orig_error=orig_error)
 
 
 class LoopbackVendor(SSHVendor):
     """SSH "vendor" that connects over a plain TCP socket, not SSH."""
-    
+
     def connect_sftp(self, username, password, host, port):
         sock = socket.socket()
         try:
@@ -263,7 +258,7 @@
 
     def _connect(self, username, password, host, port):
         global SYSTEM_HOSTKEYS, BZR_HOSTKEYS
-        
+
         load_host_keys()
 
         try:
@@ -272,36 +267,41 @@
             t.start_client()
         except (paramiko.SSHException, socket.error), e:
             self._raise_connection_error(host, port=port, orig_error=e)
-            
+
         server_key = t.get_remote_server_key()
         server_key_hex = paramiko.util.hexify(server_key.get_fingerprint())
         keytype = server_key.get_name()
         if host in SYSTEM_HOSTKEYS and keytype in SYSTEM_HOSTKEYS[host]:
             our_server_key = SYSTEM_HOSTKEYS[host][keytype]
-            our_server_key_hex = paramiko.util.hexify(our_server_key.get_fingerprint())
+            our_server_key_hex = paramiko.util.hexify(
+                our_server_key.get_fingerprint())
         elif host in BZR_HOSTKEYS and keytype in BZR_HOSTKEYS[host]:
             our_server_key = BZR_HOSTKEYS[host][keytype]
-            our_server_key_hex = paramiko.util.hexify(our_server_key.get_fingerprint())
+            our_server_key_hex = paramiko.util.hexify(
+                our_server_key.get_fingerprint())
         else:
-            warning('Adding %s host key for %s: %s' % (keytype, host, server_key_hex))
+            trace.warning('Adding %s host key for %s: %s'
+                          % (keytype, host, server_key_hex))
             add = getattr(BZR_HOSTKEYS, 'add', None)
             if add is not None: # paramiko >= 1.X.X
                 BZR_HOSTKEYS.add(host, keytype, server_key)
             else:
                 BZR_HOSTKEYS.setdefault(host, {})[keytype] = server_key
             our_server_key = server_key
-            our_server_key_hex = paramiko.util.hexify(our_server_key.get_fingerprint())
+            our_server_key_hex = paramiko.util.hexify(
+                our_server_key.get_fingerprint())
             save_host_keys()
         if server_key != our_server_key:
             filename1 = os.path.expanduser('~/.ssh/known_hosts')
-            filename2 = pathjoin(config_dir(), 'ssh_host_keys')
-            raise TransportError('Host keys for %s do not match!  %s != %s' % \
+            filename2 = osutils.pathjoin(config.config_dir(), 'ssh_host_keys')
+            raise errors.TransportError(
+                'Host keys for %s do not match!  %s != %s' %
                 (host, our_server_key_hex, server_key_hex),
                 ['Try editing %s or %s' % (filename1, filename2)])
 
-        _paramiko_auth(username, password, host, t)
+        _paramiko_auth(username, password, host, port, t)
         return t
-        
+
     def connect_sftp(self, username, password, host, port):
         t = self._connect(username, password, host, port)
         try:
@@ -331,7 +331,7 @@
 
 class SubprocessVendor(SSHVendor):
     """Abstract base class for vendors that use pipes to a subprocess."""
-    
+
     def _connect(self, argv):
         proc = subprocess.Popen(argv,
                                 stdin=subprocess.PIPE,
@@ -381,7 +381,7 @@
 
 class OpenSSHSubprocessVendor(SubprocessVendor):
     """SSH vendor that uses the 'ssh' executable from OpenSSH."""
-    
+
     def _get_vendor_specific_argv(self, username, host, port, subsystem=None,
                                   command=None):
         assert subsystem is not None or command is not None, (
@@ -426,7 +426,7 @@
         else:
             args.extend([host] + command)
         return args
-    
+
 register_ssh_vendor('ssh', SSHCorpSubprocessVendor())
 
 
@@ -454,25 +454,31 @@
 register_ssh_vendor('plink', PLinkSubprocessVendor())
 
 
-def _paramiko_auth(username, password, host, paramiko_transport):
+def _paramiko_auth(username, password, host, port, paramiko_transport):
     # paramiko requires a username, but it might be none if nothing was supplied
     # use the local username, just in case.
     # We don't override username, because if we aren't using paramiko,
     # the username might be specified in ~/.ssh/config and we don't want to
     # force it to something else
     # Also, it would mess up the self.relpath() functionality
-    username = username or getpass.getuser()
+    auth = config.AuthenticationConfig()
+    if username is None:
+        username = auth.get_user('ssh', host, port=port)
+        if username is None:
+            # Default to local user
+            username = getpass.getuser()
 
     if _use_ssh_agent:
         agent = paramiko.Agent()
         for key in agent.get_keys():
-            mutter('Trying SSH agent key %s' % paramiko.util.hexify(key.get_fingerprint()))
+            trace.mutter('Trying SSH agent key %s'
+                         % paramiko.util.hexify(key.get_fingerprint()))
             try:
                 paramiko_transport.auth_publickey(username, key)
                 return
             except paramiko.SSHException, e:
                 pass
-    
+
     # okay, try finding id_rsa or id_dss?  (posix only)
     if _try_pkey_auth(paramiko_transport, paramiko.RSAKey, username, 'id_rsa'):
         return
@@ -487,14 +493,12 @@
             pass
 
     # give up and ask for a password
-    password = bzrlib.ui.ui_factory.get_password(
-            prompt='SSH %(user)s@%(host)s password',
-            user=username, host=host)
+    password = auth.get_password('ssh', host, username, port=port)
     try:
         paramiko_transport.auth_password(username, password)
     except paramiko.SSHException, e:
-        raise ConnectionError('Unable to authenticate to SSH host as %s@%s' %
-                              (username, host), e)
+        raise errors.ConnectionError(
+            'Unable to authenticate to SSH host as %s@%s' % (username, host), e)
 
 
 def _try_pkey_auth(paramiko_transport, pkey_class, username, filename):
@@ -504,17 +508,18 @@
         paramiko_transport.auth_publickey(username, key)
         return True
     except paramiko.PasswordRequiredException:
-        password = bzrlib.ui.ui_factory.get_password(
-                prompt='SSH %(filename)s password',
-                filename=filename)
+        password = ui.ui_factory.get_password(
+            prompt='SSH %(filename)s password', filename=filename)
         try:
             key = pkey_class.from_private_key_file(filename, password)
             paramiko_transport.auth_publickey(username, key)
             return True
         except paramiko.SSHException:
-            mutter('SSH authentication via %s key failed.' % (os.path.basename(filename),))
+            trace.mutter('SSH authentication via %s key failed.'
+                         % (os.path.basename(filename),))
     except paramiko.SSHException:
-        mutter('SSH authentication via %s key failed.' % (os.path.basename(filename),))
+        trace.mutter('SSH authentication via %s key failed.'
+                     % (os.path.basename(filename),))
     except IOError:
         pass
     return False
@@ -527,14 +532,15 @@
     """
     global SYSTEM_HOSTKEYS, BZR_HOSTKEYS
     try:
-        SYSTEM_HOSTKEYS = paramiko.util.load_host_keys(os.path.expanduser('~/.ssh/known_hosts'))
+        SYSTEM_HOSTKEYS = paramiko.util.load_host_keys(
+            os.path.expanduser('~/.ssh/known_hosts'))
     except IOError, e:
-        mutter('failed to load system host keys: ' + str(e))
-    bzr_hostkey_path = pathjoin(config_dir(), 'ssh_host_keys')
+        trace.mutter('failed to load system host keys: ' + str(e))
+    bzr_hostkey_path = osutils.pathjoin(config.config_dir(), 'ssh_host_keys')
     try:
         BZR_HOSTKEYS = paramiko.util.load_host_keys(bzr_hostkey_path)
     except IOError, e:
-        mutter('failed to load bzr host keys: ' + str(e))
+        trace.mutter('failed to load bzr host keys: ' + str(e))
         save_host_keys()
 
 
@@ -543,8 +549,8 @@
     Save "discovered" host keys in $(config)/ssh_host_keys/.
     """
     global SYSTEM_HOSTKEYS, BZR_HOSTKEYS
-    bzr_hostkey_path = pathjoin(config_dir(), 'ssh_host_keys')
-    ensure_config_dir_exists()
+    bzr_hostkey_path = osutils.pathjoin(config.config_dir(), 'ssh_host_keys')
+    config.ensure_config_dir_exists()
 
     try:
         f = open(bzr_hostkey_path, 'w')
@@ -554,7 +560,7 @@
                 f.write('%s %s %s\n' % (hostname, keytype, key.get_base64()))
         f.close()
     except IOError, e:
-        mutter('failed to save bzr host keys: ' + str(e))
+        trace.mutter('failed to save bzr host keys: ' + str(e))
 
 
 def os_specific_subprocess_params():

=== modified file 'doc/developers/authentication-ring.txt'
--- doc/developers/authentication-ring.txt	2007-09-19 09:38:34 +0000
+++ doc/developers/authentication-ring.txt	2007-11-04 15:24:27 +0000
@@ -70,7 +70,8 @@
 Authentication definitions
 --------------------------
 
-There are two kinds of authentication used by the various schemes:
+There are two kinds of authentication used by the various schemes supported by
+bzr:
 
 1. user and password
 
@@ -101,20 +102,25 @@
 ``password``).
 
   * ``scheme``: can be empty (meaning the rest of the definition can be used
-    for any scheme),
+    for any scheme), ``SFTP`` and ``bzr+ssh`` should not be used here, ``ssh``
+    should be used instead since this is the real scheme regarding
+    authentication,
 
   * ``host``: can be empty (to act as a default for any host),
 
   * ``port`` can be empty (useful when an host provides several servers for the
-    same scheme), only numerical values are allowed,
+    same scheme), only numerical values are allowed, this should be used only
+    when the server uses a port different than the scheme standard port,
 
   * ``path``: can be empty (FTP or SFTP will never user it),
 
-  * ``user``: can be empty (and will use the same default as ``bzr``, python's
-    ``getpass.get_user()``),
+  * ``user``: can be empty (``bzr`` will defaults to python's
+    ``getpass.get_user()`` and attempt another matching(see below)),
 
   * ``password``: can be empty (for security reasons, a user may use the
-    definitions without storing the passwords but want to be prompted).
+    definitions without storing the passwords but want to be prompted ; or the
+    password will be provided by an external plugin via the
+    ``password_encoding`` mechanism decribed below).
 
   * ``password_encoding``: can be empty (default is ``plaintext``).
 
@@ -131,8 +137,9 @@
 
  3. ``scheme`` matches even if decorators are used in the requested URL,
 
- 4. ``host`` matches if included in the requested URL. ``foo.net`` will match a
-    requested ``bzr.foo.net``.
+ 4. ``host`` matches exactly or act as a domain if it starts with '.'
+    (``project.bzr.sf.net`` will match ``.bzr.sf.net`` but ``projectbzr.sf.net``
+    will not match ``bzr.sf.net``).
 
  5. ``port`` matches if included in the requested URL (exact matches only)
 
@@ -158,9 +165,10 @@
 
 .. [#password_encoding] Additional password encoding methods may be defined
    that will rely on external means to store the password which, in these
-   cases, will not appear anymore in the definition but be provided. An
-   encoding named ``netrc`` for example will provide passwords by using the
-   ``.netrc`` file.
+   cases, will not appear anymore in the definition. It is assumed that
+   additional password encodings will provide a storage outside of the file
+   described here. An encoding named ``netrc`` for example will provide
+   passwords by retrieving them in the ``.netrc`` file.
 
 File format
 -----------
@@ -181,7 +189,7 @@
 ``~/.bazaar/authentication.conf`` will use the same file format than
 ``~/.bazaar/bazaar.conf``.
 
-Each section will define an authentication definition.
+Each section describes an authentication definition.
 
 The section name is an arbitrary string, only the ``DEFAULT`` value is reserved
 and should appear as the *last* section.
@@ -212,7 +220,8 @@
 
 This section could define:
 
-  * ``user``: default user to be used.
+  * ``user``: default user to be used (if not defined the usual
+    bzr way applies, see below).
 
   * ``password_encoding``: default password encoding.
 
@@ -236,28 +245,30 @@
         bzr branch ftp://foo.net/bzr/branch
         bzr pull ftp://bzr.foo.net/bzr/product/branch/trunk
 
-  * all connections are done with the same ``user`` (the local one) and the
-    password is always prompted with some exceptions::
+  * all connections are done with the same ``user`` (the remote one for which
+    the default bzr one is not appropriate) and the password is always prompted
+    with some exceptions::
 
         # Pet projects on hobby.net
         [hobby]
-        scheme=https
         host=r.hobby.net
-        verify_certificates=no
         user=jim
         password=obvious1234
         
         # Home server
         [home]
+        scheme=https
         host=home.net
         user=joe
         password='c2VjcmV0LXBhc3M='
         password_encoding=base64
+        verify_certificates=no # Still searching a free certificate provider
         
         [DEFAULT]
+        # Our local user is barbaz, on all remote sites we're known as foobar
         user=foobar
         
-  * an HTTP server that also acts as a proxy (weird)::
+  * an HTTP server and a proxy::
 
         # development branches on dev server
         [dev]
@@ -275,15 +286,24 @@
         user=user2
         password=pass2
         
-        # Pesky proxy
+        # proxy
         [proxy]
         scheme=http
-        host=dev.company.com
+        host=proxy.company.com
+        port=3128
         user=proxyuser1
         password=proxypass1
-        
-Note that the proxy should be specified last because it uses no path. An
-alternative is to specify the port used by the proxy.
+
+  * source hosting provider declaring sub-domains for each project::
+
+        [sfnet domain]
+        # we use sftp, but ssh is the scheme used for authentication
+        scheme=ssh
+        # The leading '.' ensures that 'sf.net' alone doesn't match
+        host=.sf.net
+        user=georges
+        password=ben...son
+
 
 UI Changes
 ----------
@@ -305,8 +325,8 @@
   Get a user from ``~/.bazaar/authentication.conf`` or prompt for one if none is
   found. Continue as 2.
 
-Note: A user will be queried only if the server requires it for ``HTTP``, other
-protocols always require a user.
+Note: A user will be queried only if the server requires it for ``HTTP`` or
+``HTTPS``, other protocols always require a user.
 
 In any case, if the server refuses the authentication, bzr reports to the user
 and terminates.
@@ -315,7 +335,7 @@
 --------------------------
 
 * bzr should be able to prompt for a ``user`` for a given (``scheme``, ``host``
-  [, ``realm``]). Note that ``realm`` may be available only after a first
+  [, ``realm``]). Note that ``realm`` is available only after a first
   connection attempt to the server.
 
 * No assumptions should be made about the clients of this service

=== added file 'doc/en/user-guide/authentication_conf.txt'
--- doc/en/user-guide/authentication_conf.txt	1970-01-01 00:00:00 +0000
+++ doc/en/user-guide/authentication_conf.txt	2007-11-04 15:24:27 +0000
@@ -0,0 +1,227 @@
+=================================
+Authentication configuration file
+=================================
+
+
+Intent
+======
+
+Many different authentication policies can be described in the
+``$HOME/.bazaar/authentication.conf`` file but a particular user should need
+only a few definitions to cover his needs without having to specify a user and
+a password for every branch he uses.
+
+The definitions found in this file are used to find the credentials to use for
+a given url. The same credentials can generally be used for as many branches as
+possible by grouping their declaration around the remote servers that need
+them. It's even possible to declare credentials that will be used by different
+servers.
+
+The intent is to make this file as small as possible to minimize maintenance.
+
+Once the relevant credentials are declared in this file you may use branch urls
+without embedding passwords (security hazard) or even users (enabling sharing
+of your urls with others).
+
+Instead of using::
+
+  bzr branch ftp://joe:secret@host.com/path/to/my/branch
+
+you simply use::
+
+  bzr branch ftp://host.com/path/to/my/branch
+
+provided you have created the following ``authentication.conf`` file::
+
+  [myprojects]
+  scheme=ftp
+  host=host.com
+  user=joe
+  password=secret
+  
+
+Authentication definitions
+==========================
+
+There are two kinds of authentication used by the various schemes supported by
+bzr:
+
+1. user and password
+
+``FTP`` and ``SFTP`` needs a (``user``, ``password``) to authenticate against a
+``host`` (SFTP can use ssh keys too, but we don't talk about that here as ssh
+agents provide a better solution).
+
+2. user, realm and password
+
+``HTTP`` and ``HTTPS`` needs a (``user, realm, password``) to authenticate
+against a host. But, by using ``.htaccess`` files, for example, it is possible
+to define several (``user, realm, password``) for a given ``host``. So what is
+really needed is (``user``, ``password``, ``host``, ``path``). The ``realm`` is
+not taken into account in the defitions, but will displayed if bzr prompts you
+for a password.
+
+``HTTP proxy`` can be handled as ``HTTP`` (or ``HTTPS``) by explicitely
+specifying the appropriate port.
+
+To take all schemes into account, the password will be deduced from a set of
+authentication definitions (``scheme``, ``host``, ``port``, ``path``, ``user``,
+``password``).
+
+  * ``scheme``: can be empty (meaning the rest of the definition can be used
+    for any scheme), ``SFTP`` and ``bzr+ssh`` should not be used here, ``ssh``
+    should be used instead since this is the real scheme regarding
+    authentication,
+
+  * ``host``: can be empty (to act as a default for any host),
+
+  * ``port`` can be empty (useful when an host provides several servers for the
+    same scheme), only numerical values are allowed, this should be used only
+    when the server uses a port different than the scheme standard port,
+
+  * ``path``: can be empty (FTP or SFTP will never user it),
+
+  * ``user``: can be empty (``bzr`` will defaults to python's
+    ``getpass.get_user()``),
+
+  * ``password``: can be empty if you prefer to always be prompted for your
+    password.
+
+Multiple definitions can be provided and, for a given URL, bzr will select a
+(``user`` [, ``password``]) based on the following rules :
+
+ 1. the first match wins,
+
+ 2. empty fields match everything,
+
+ 3. ``scheme`` matches even if decorators are used in the requested URL,
+
+ 4. ``host`` matches exactly or act as a domain if it starts with '.'
+    (``project.bzr.sf.net`` will match ``.bzr.sf.net`` but ``projectbzr.sf.net``
+    will not match ``bzr.sf.net``).
+
+ 5. ``port`` matches if included in the requested URL (exact matches only)
+
+ 6. ``path`` matches if included in the requested URL (and by rule #2 above,
+    empty paths will match any provided path).
+
+
+
+File format
+===========
+
+The general rules for `configuration files`_ apply except for the variable
+policies.
+
+.. _configuration files: configuration.html
+
+Each section describes an authentication definition.
+
+The section name is an arbitrary string, only the ``DEFAULT`` value is reserved
+and should appear as the *last* section.
+
+Each section should define:
+
+* ``user``: the login to be used,
+
+Each section could define:
+
+* ``host``: the remote server,
+
+* ``port``: the port the server is listening,
+
+* ``path``: the branch location,
+
+* ``password``: the password.
+
+
+Examples
+========
+
+
+Personal projects hosted outside
+--------------------------------
+
+All connections are done with the same ``user`` (the remote one for which the
+default bzr one is not appropriate) and the password is always prompted with
+some exceptions::
+
+        # Pet projects on hobby.net
+        [hobby]
+        host=r.hobby.net
+        user=jim
+        password=obvious1234
+        
+        # Home server
+        [home]
+        scheme=https
+        host=home.net
+        user=joe
+        password=1essobV10us
+        
+        [DEFAULT]
+        # Our local user is barbaz, on all remote sites we're known as foobar
+        user=foobar
+
+
+Source hosting provider
+-----------------------
+
+In the shp.net (fictious) domain, each project has its own site::
+
+        [shpnet domain]
+        # we use sftp, but ssh is the scheme used for authentication
+        scheme=ssh
+        # The leading '.' ensures that 'shp.net' alone doesn't match
+        host=.shp.net
+        user=joe
+        password=precious
+
+HTTPS, SFTP servers and their proxy
+-----------------------------------
+
+At company.com, the server hosting released and integration code is behind a
+proxy, the two servers use different authentication policies::
+
+        [reference code]
+        scheme=https
+        host=dev.company.com
+        path=/dev
+        user=user1
+        password=pass1
+
+        # development branches on dev server
+        [dev]
+        scheme=ssh # bzr+ssh and sftp are available here
+        host=dev.company.com
+        path=/dev/integration
+        user=user2
+        password=pass2
+        
+        # proxy
+        [proxy]
+        scheme=http
+        host=proxy.company.com
+        port=3128
+        user=proxyuser1
+        password=proxypass1
+
+
+Planned enhancements
+====================
+
+The following are not yet implemented but planned as parts of a work in
+progress:
+
+* add a  ``password_encoding`` field allowing:
+
+  - storing the passwords in various obfuscating encodings (base64 for one),
+
+  - delegate password storage to plugins (.netrc for example).
+
+* update the credentials when the user is prompted for user or password,
+
+* add a ``verify_certificates`` field for ``HTTPS``.
+
+The ``password_encoding`` and ``verify_certificates`` fields are recognized but
+ignored in the actual implementation.
\ No newline at end of file

=== modified file 'doc/en/user-guide/configuration.txt'
--- doc/en/user-guide/configuration.txt	2007-10-19 13:51:48 +0000
+++ doc/en/user-guide/configuration.txt	2007-11-04 15:24:27 +0000
@@ -64,12 +64,22 @@
 The path where Bazaar should look for shell plugin external commands.
 
 
-Location of configuration files
-===============================
-Each user gets a pair of configurations files in ``$HOME/.bazaar``. The first
-one, named ``bazaar.conf``, includes default configuration options. The other
-file, ``locations.conf``, contains configuration information for specific
-branch locations.  These files are sometimes referred to as ``ini files``.
+Configuration files
+===================
+
+Location
+--------
+
+Configuration files are located in ``$HOME/.bazaar`` and are
+sometimes referred to as ``ini files``:
+
+* ``bazaar.conf`` describes default configuration options,
+
+* ``locations.conf`` describes configuration information for
+  specific branch locations,
+
+* ``authentication.conf`` describes credential information for
+  remote servers.
 
 Each branch can also contain a configuration file that sets values specific
 to that branch. This file is found at ``.bzr/branch/branch.conf`` within the
@@ -78,18 +88,19 @@
 can do so in ``locations.conf``.
 
 General Format
-==============
+--------------
+
 An ini file has three types of contructs: section headers, section
 variables and comments.
 
 Comments
---------
+~~~~~~~~
 A comment is any line that starts with a "#" (sometimes called a "hash
 mark", "pound sign" or "number sign"). Comment lines are ignored by
 Bazaar when parsing ini files.
 
 Section Headers
----------------
+~~~~~~~~~~~~~~~
 A section header is a word enclosed in brackets that starts at the begining
 of a line.  A typical section header looks like this::
 
@@ -109,7 +120,7 @@
 
 
 Section Variables
------------------
+~~~~~~~~~~~~~~~~~
 
 A section variable resides within a section. A section variable contains a
 variable name, an equals sign and a value.  For example::
@@ -119,7 +130,7 @@
 
 
 Variable Policies
------------------
+~~~~~~~~~~~~~~~~~
 
 Variables defined in a section affect the named directory or URL plus
 any locations they contain.  Policies can be used to change how a
@@ -164,11 +175,15 @@
     check_signatures  = check-available
     create_signatures = when-required
 
-``$HOME/.bazaar/locations.conf`` allows one to specify overriding settings for a
-specific branch. The format is almost identical to the default section in
-bazaar.conf with one significant change: The section header, instead of
-saying default, will be the path to a branch that you wish to override a
-value for. The '?' and '*' wildcards are supported::
+
+The branch location configuration file, locations.conf
+------------------------------------------------------
+
+``$HOME/.bazaar/locations.conf`` allows one to specify overriding settings for
+a specific branch. The format is almost identical to the default section in
+bazaar.conf with one significant change: The section header, instead of saying
+default, will be the path to a branch that you wish to override a value
+for. The '?' and '*' wildcards are supported::
 
     [/home/jdoe/branches/nethack]
     email = Nethack Admin <nethack@nethack.com>
@@ -180,6 +195,21 @@
     [http://bazaar-vcs.org/bzr/*]
     check_signatures  = require
 
+The authentication configuration file, authentication.conf
+----------------------------------------------------------
+
+``$HOME/.bazaar/authentication.conf`` allows one to specify credentials for
+remote servers. This can be used for all the supported transports and any part
+of bzr that requires authentication (smtp for example).
+
+The syntax of the file obeys the same rules as the others except for the
+variable policies which don't apply.
+
+For more information on the possible uses of the authentication configuration
+file see the `authentication configuration file documentation`_.
+
+.. _authentication configuration file documentation: authentication_conf.html
+
 Common Variable Options
 =======================
 

